// parts of this file are based on work from pan One (http://home-3.tiscali.nl/~meost/pms/)

//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "knownfile.h"
#include "opcodes.h"
#include <io.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "emule.h"
#include "ini2.h"

#if JUMPSTART
#include "Jumpstarter.h"
#endif

	
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//--- xrmb:parttraffic ---
//--- i don't like code in header files ---
CFileStatistic::CFileStatistic()
{
	EMULE_TRY

	partTraffic=NULL;
	partTrafficSession=NULL;

	blockTraffic=NULL;
	blockTrafficSession=NULL;

	partAccepted=NULL;
	partAcceptedSession=NULL;

	completeReleases=0.0;
	requested = transfered = accepted = alltimerequested= alltimetransfered = alltimeaccepted = 0;

	EMULE_CATCH
}

CFileStatistic::~CFileStatistic()	
{
	EMULE_TRY

	if(partTraffic)				delete[] partTraffic;			partTraffic=NULL;
	if(partTrafficSession)		delete[] partTrafficSession;	partTrafficSession=NULL;

	if(partAccepted)			delete[] partAccepted;			partAccepted=NULL;
	if(partAcceptedSession)		delete[] partAcceptedSession;	partAcceptedSession=NULL;

	if(blockTraffic)			delete[] blockTraffic;			blockTraffic=NULL;
	if(blockTrafficSession)		delete[] blockTrafficSession;	blockTrafficSession=NULL;

	EMULE_CATCH
}
//--- :xrmb ---

void CFileStatistic::AddRequest()
{
	EMULE_TRY

	requested++;
	alltimerequested++;
	theApp.knownfiles->requested++;
	theApp.sharedfiles->UpdateItem(fileParent);

	EMULE_CATCH
}
	
void CFileStatistic::AddAccepted()
{
	EMULE_TRY

	accepted++;
	alltimeaccepted++;
	theApp.knownfiles->accepted++;
	theApp.sharedfiles->UpdateItem(fileParent);

	EMULE_CATCH
}
	
void CFileStatistic::AddTransferd(uint32 bytes)
{
	EMULE_TRY

	transfered += bytes;
	alltimetransfered += bytes;
	theApp.knownfiles->transfered += bytes;
	theApp.sharedfiles->UpdateItem(fileParent);

	EMULE_CATCH
}

CKnownFile::CKnownFile()
{
	filetype = 2;
	directory = 0;
	filename = 0;
	filesize = 0;
	date = 0;
	m_iPriority = PR_NORMAL;
	m_iPermissions = PERM_ALL;
	statistic.fileParent=this;
	m_bAutoPriority = false ;
	m_bCommentLoaded=false;
	m_iMoviePreviewMode=0; // DonGato - Moview Preview Mode
	m_iRate=0;
	m_strComment="";
	hashlist.RemoveAll();	// just in case
	taglist.RemoveAll();	// you never know

	//--- xrmb:allyaknow ---
	m_sharedFile=false;
	//--- :xrmb ---
	memset(filehash, 0, 16);

#if JUMPSTART
    m_Jumpstarter = NULL;
#endif
}

CKnownFile::~CKnownFile()
{
	EMULE_TRY

	for (int i = 0; i != hashlist.GetSize(); i++)
		if (hashlist[i])
			delete[] hashlist[i];
	for (int i = 0; i != taglist.GetSize(); i++)
		safe_delete(taglist[i]);
//	if (filename)	// done by CAbstractFile destructor
//		delete[] filename;
	if (directory)
		delete[] directory;

	EMULE_CATCH
}

void CKnownFile::SetPath(char* path)
{
	EMULE_TRY

	if (directory)
		delete[] directory;
	directory = nstrdup(path);

	EMULE_CATCH
}

bool CKnownFile::CreateFromFile(char* in_directory,char* in_filename)
{
	EMULE_TRY

	// TODO Errorhandling
	//first create the filehashset
	// open file
	directory = nstrdup(in_directory);

	char* namebuffer = new char[strlen(in_directory)+strlen(in_filename)+2];
	sprintf(namebuffer,"%s\\%s",in_directory,in_filename);
	FILE* file = fopen(namebuffer,"rbS");
	delete namebuffer;
	if (!file)
		return false;
	// set filesize + name
	filename = nstrdup(in_filename);
	filesize =_filelength(file->_file);
	// create hashset
	uint32 togo = filesize;
	for (uint16 hashcount = 0;togo >= PARTSIZE;) {
		uchar* newhash = new uchar[16];
		CreateHashFromFile(file,PARTSIZE,newhash);
		hashlist.Add(newhash);
		togo -= PARTSIZE;
		hashcount++;
	}
	uchar* lasthash = new uchar[16];
	memset(lasthash,0,16);
	CreateHashFromFile(file,togo,lasthash);
	if (!hashcount)
	{
		memcpy(filehash,lasthash,16);
        	delete[] lasthash; // i_a: memleak
	}
	else 
	{
		hashlist.Add(lasthash);
		uchar* buffer = new uchar[hashlist.GetCount()*16];
		for (int i = 0;i != hashlist.GetCount();i++)
			memcpy(buffer+(i*16),hashlist[i],16);
		CreateHashFromString(buffer,hashlist.GetCount()*16,filehash);
		delete[] buffer;
	}
	// TODO: Add filetags

	// set lastwrite date
	struct _stat fileinfo;
	_fstat(file->_file,&fileinfo);
	date = fileinfo.st_mtime;
	//finished
	fclose(file);
	return true;	

	EMULE_CATCH

	return false;
}


// needed for memfiles. its probably better to switch everything to CFile...
bool CKnownFile::LoadHashsetFromFile(CFile* file, bool checkhash)
{
	EMULE_TRY

	uchar checkid[16];
	file->Read(&checkid,16);
	uint16	parts;
	file->Read(&parts,2);
	for (int i = 0; i != parts; i++){
		uchar* cur_hash = new uchar[16];
		file->Read(cur_hash,16);
		hashlist.Add(cur_hash);
	}
	if (!checkhash){
		memcpy(filehash,checkid,16);
		// return true;  // obaldin - do _some_ check anyways
	}
	// trust noone ;-)

	if (!hashlist.IsEmpty()){
		uchar* buffer = new uchar[hashlist.GetCount()*16];
		for (int i = 0;i != hashlist.GetCount();i++)
			memcpy(buffer+(i*16),hashlist[i],16);
		CreateHashFromString(buffer,hashlist.GetCount()*16,checkid);
		delete[] buffer;
	}
	if (!memcmp(filehash,checkid,16))
		return true;
	else
	{
		for (int i = 0; i != hashlist.GetSize(); i++)
		{	
			delete[] hashlist[i];
			hashlist[i]= NULL;	// most likely this is needless, but on the other hand...
		}
		hashlist.RemoveAll();
		return false;
	}

	EMULE_CATCH

	return false;
}

bool CKnownFile::LoadTagsFromFile(CFile* file)
{
	EMULE_TRY

	uint32 tagcount;
	file->Read(&tagcount,4);
	for (uint32 j = 0; j != tagcount;j++){
		CTag* newtag = new CTag(file);
		switch(newtag->tag->specialtag){
			case FT_FILENAME:{
				filename = nstrdup(newtag->tag->stringvalue);
				delete newtag;
				break;
			}
			case FT_FILESIZE:{
				filesize = newtag->tag->intvalue;
				delete newtag;
				break;
			}
			case FT_ATTRANSFERED:{
				statistic.alltimetransfered = newtag->tag->intvalue;
				delete newtag;
				break;
			}
			case FT_ATREQUESTED:{
				statistic.alltimerequested = newtag->tag->intvalue;
				delete newtag;
				break;
			}
 			case FT_ATACCEPTED:{
				statistic.alltimeaccepted = newtag->tag->intvalue;
				delete newtag;
				break;
			}
			case FT_PRIORITY:{
				m_iPriority = newtag->tag->intvalue;
				if (m_iPriority == PR_AUTO) {
					SetAutoPriority(true);
					UpdateUploadAutoPriority();
				}
				delete newtag;
				break;
			}
			case FT_PERMISSIONS:{
				m_iPermissions = newtag->tag->intvalue;
				delete newtag;
				break;
			}
			default:
				taglist.Add(newtag);
		}	
	}
	return true;

	EMULE_CATCH

	return false;
}

bool CKnownFile::LoadDateFromFile(CFile* file)
{
	EMULE_TRY

	file->Read(&date,4);
	return true;

	EMULE_CATCH

	return false;
}

bool CKnownFile::LoadFromFile(CFile* file)
{
	EMULE_TRY

    bool result;
    bool hashset_ok = true;

    result = LoadDateFromFile(file);
    hashset_ok = LoadHashsetFromFile(file,false);
    result = result && LoadTagsFromFile(file);
    result = result && hashset_ok;

    if(!hashset_ok) {
        theApp.emuledlg->AddLogLine(false,"Loaded hashset is corrupt in known.met, file: %s!", filename);
    }

#if JUMPSTART
    if(result) {
        if(CJumpstarter::ShouldBeEnabledForFile(this)) {
            m_Jumpstarter = new CJumpstarter(this);
            theApp.emuledlg->AddLogLine(false,"Jumpstart enabled for file: %s!", filename);
        }
    }
#endif // JUMPSTART

	return result;

    EMULE_CATCH

	return false;
}

bool CKnownFile::WriteToFile(FILE* file)
{
	EMULE_TRY

	// date
	fwrite(&date,4,1,file); 
	// hashset
	fwrite(&filehash,16,1,file);
	uint16 parts = hashlist.GetCount();
	fwrite(&parts,2,1,file);
	for (int i = 0; i != parts; i++)
	{	
		if(hashlist[i] == NULL)
			return false; // ? is this correct? i hope it doesn't corrupt the hashsets.
		fwrite(hashlist[i],16,1,file);
	}
	//tags
	uint32 tagcount = taglist.GetCount()+7;
	// standard tags
	fwrite(&tagcount,4,1,file);
	CTag* nametag = new CTag(FT_FILENAME,filename);
	nametag->WriteTagToFile(file);
	delete nametag;
	CTag* sizetag = new CTag(FT_FILESIZE,filesize);
	sizetag->WriteTagToFile(file);
	delete sizetag;
	// statistic
	CTag attag1(FT_ATTRANSFERED,statistic.GetAllTimeTransfered());
	attag1.WriteTagToFile(file);
	CTag attag2(FT_ATREQUESTED,statistic.GetAllTimeRequests());
	attag2.WriteTagToFile(file);
	CTag attag3(FT_ATACCEPTED,statistic.GetAllTimeAccepts());
	attag3.WriteTagToFile(file);
	// priority N permission
	CTag* priotag;
	if (IsPartFile() && IsAutoPrioritized())
		priotag = new CTag(FT_PRIORITY, (uint8)PR_AUTO);
	else
		priotag = new CTag(FT_PRIORITY, m_iPriority);
	priotag->WriteTagToFile(file);
	delete priotag;
	CTag permtag(FT_PERMISSIONS, m_iPermissions);
	permtag.WriteTagToFile(file);
		//other tags
	for (uint32 j = 0; j != tagcount-7;j++)
		taglist[j]->WriteTagToFile(file);
	return ferror(file);

	EMULE_CATCH

	return false;
}

void CKnownFile::CreateHashFromInput(FILE* file,CFile* file2, int Length, uchar* Output, uchar* in_string) 
{
	EMULE_TRY

	// time critial
	bool PaddingStarted = false;
	uint32 Hash[4];
	Hash[0] = 0x67452301;
	Hash[1] = 0xEFCDAB89;
	Hash[2] = 0x98BADCFE;
	Hash[3] = 0x10325476;
	CFile* data = 0;
	if (in_string)
		data = new CMemFile(in_string,Length);
	uint32 Required = Length;
	uchar   X[64*128];
	while (Required >= 64){
		uint32 len = Required / 64; 
		if (len > sizeof(X)/(64 * sizeof(X[0]))) 
			len = sizeof(X)/(64 * sizeof(X[0])); 
		if (in_string)
			data->Read(&X,len*64);
		else if (file)
			fread(&X,len*64,1,file);
		else if (file2)
			file2->Read(&X,len*64); 
		for (uint32 i = 0; i < len; i++)
		{ 
			MD4Transform(Hash, (uint32*)(X + i*64)); 
		} 
		Required -= len*64; 
	}
	// bytes to read
	Required = Length % 64;
	if (Required != 0){
		if (in_string)
			data->Read(&X,Required);
		else if (file)
			fread(&X,Required,1,file);
		else if (file2)
			file2->Read(&X,Required);
	}
	// in byte scale 512 = 64, 448 = 56
	if (Required >= 56){
		X[Required] = 0x80;
		PaddingStarted = TRUE;
		memset(&X[Required + 1], 0, 63 - Required);
		MD4Transform(Hash, (uint32*)X);
		Required = 0;
	}
	if (!PaddingStarted)
		X[Required++] = 0x80;
	memset(&X[Required], 0, 64 - Required);
	// add size (convert to bits)
	uint32 Length2 = Length >> 29;
	Length <<= 3;
	memcpy(&X[56], &Length, 4);
	memcpy(&X[60], &Length2, 4);
	MD4Transform(Hash, (uint32*)X);
	memcpy(Output, Hash, 16);
	safe_delete(data);

	EMULE_CATCH
}
uchar* CKnownFile::GetPartHash(uint16 part)
{
	EMULE_TRY

	if (part >= hashlist.GetCount())
		return 0;
	return hashlist[part];

	EMULE_CATCH

	return 0;
}

// Added by Tarod to push RF files
float CKnownFile::GetFileRatio()
{
	EMULE_TRY

	// MatzeHH formula I think is better than herbert's one (Tarod)
	if (theApp.knownfiles->GetTotalRequested()==0) return 1.0f;	//Cax2 bugfix
	if (statistic.GetRequests() == 0) return 2.0f ;
	float ratio;
	ratio = ((float)statistic.GetRequests()) / ((float)theApp.knownfiles->GetTotalRequested()) ;
	ratio = -4.0f * log10f(((float)theApp.knownfiles->GetCount())) * ratio ;
	ratio = 2.0f * expf(ratio) ;

	if (ratio < 1.0f) {
		ratio = 1.0f;
	} else if (ratio > 2.0f)
		ratio = 2.0f ;

	return ratio ;

	EMULE_CATCH

	return 1.0f;
}
// End by Tarod

uint16 CKnownFile::GetPartCount()
{
	EMULE_TRY

	return (filesize/PARTSIZE) + ((filesize % PARTSIZE)? 1:0);

	EMULE_CATCH

	return 0;
}

static void MD4Transform(uint32 Hash[4], uint32 x[16])
{
	EMULE_TRY

	uint32 a = Hash[0];
	uint32 b = Hash[1];
	uint32 c = Hash[2];
	uint32 d = Hash[3];

	/* Round 1 */
	MD4_FF(a, b, c, d, x[ 0], S11); // 01
	MD4_FF(d, a, b, c, x[ 1], S12); // 02
	MD4_FF(c, d, a, b, x[ 2], S13); // 03
	MD4_FF(b, c, d, a, x[ 3], S14); // 04
	MD4_FF(a, b, c, d, x[ 4], S11); // 05
	MD4_FF(d, a, b, c, x[ 5], S12); // 06
	MD4_FF(c, d, a, b, x[ 6], S13); // 07
	MD4_FF(b, c, d, a, x[ 7], S14); // 08
	MD4_FF(a, b, c, d, x[ 8], S11); // 09
	MD4_FF(d, a, b, c, x[ 9], S12); // 10
	MD4_FF(c, d, a, b, x[10], S13); // 11
	MD4_FF(b, c, d, a, x[11], S14); // 12
	MD4_FF(a, b, c, d, x[12], S11); // 13
	MD4_FF(d, a, b, c, x[13], S12); // 14
	MD4_FF(c, d, a, b, x[14], S13); // 15
	MD4_FF(b, c, d, a, x[15], S14); // 16

	/* Round 2 */
	MD4_GG(a, b, c, d, x[ 0], S21); // 17
	MD4_GG(d, a, b, c, x[ 4], S22); // 18
	MD4_GG(c, d, a, b, x[ 8], S23); // 19
	MD4_GG(b, c, d, a, x[12], S24); // 20
	MD4_GG(a, b, c, d, x[ 1], S21); // 21
	MD4_GG(d, a, b, c, x[ 5], S22); // 22
	MD4_GG(c, d, a, b, x[ 9], S23); // 23
	MD4_GG(b, c, d, a, x[13], S24); // 24
	MD4_GG(a, b, c, d, x[ 2], S21); // 25
	MD4_GG(d, a, b, c, x[ 6], S22); // 26
	MD4_GG(c, d, a, b, x[10], S23); // 27
	MD4_GG(b, c, d, a, x[14], S24); // 28
	MD4_GG(a, b, c, d, x[ 3], S21); // 29
	MD4_GG(d, a, b, c, x[ 7], S22); // 30
	MD4_GG(c, d, a, b, x[11], S23); // 31
	MD4_GG(b, c, d, a, x[15], S24); // 32

	/* Round 3 */
	MD4_HH(a, b, c, d, x[ 0], S31); // 33
	MD4_HH(d, a, b, c, x[ 8], S32); // 34
	MD4_HH(c, d, a, b, x[ 4], S33); // 35
	MD4_HH(b, c, d, a, x[12], S34); // 36
	MD4_HH(a, b, c, d, x[ 2], S31); // 37
	MD4_HH(d, a, b, c, x[10], S32); // 38
	MD4_HH(c, d, a, b, x[ 6], S33); // 39
	MD4_HH(b, c, d, a, x[14], S34); // 40
	MD4_HH(a, b, c, d, x[ 1], S31); // 41
	MD4_HH(d, a, b, c, x[ 9], S32); // 42
	MD4_HH(c, d, a, b, x[ 5], S33); // 43
	MD4_HH(b, c, d, a, x[13], S34); // 44
	MD4_HH(a, b, c, d, x[ 3], S31); // 45
	MD4_HH(d, a, b, c, x[11], S32); // 46
	MD4_HH(c, d, a, b, x[ 7], S33); // 47
	MD4_HH(b, c, d, a, x[15], S34); // 48

	Hash[0] += a;
	Hash[1] += b;
	Hash[2] += c;
	Hash[3] += d;

	EMULE_CATCH
}

// Updates priority of file if autopriority is activated
void CKnownFile::UpdateUploadAutoPriority(void)
{
	EMULE_TRY

	if (IsAutoPrioritized() && theApp.downloadqueue )
	{
		CPartFile* partfile = theApp.downloadqueue->GetFileByID(GetFileHash()) ;
		
		if (partfile != NULL && partfile->IsPartFile()) {
			int valid_sources = partfile->GetSourceCount() - partfile->GetNotCurrentSourcesCount();
			if (valid_sources < RARE_FILE)
				SetPriority(PR_VERYHIGH);
			else if (valid_sources < 200)
				SetPriority(PR_HIGH);
			else
				SetPriority(PR_NORMAL);
		}
	}

	EMULE_CATCH
}

// Adde by Tarod [Juanjo]
void CAbstractFile::SetFileName(char* NewName) 
{
	EMULE_TRY

	if (filename!=NULL) { 
		delete[] filename; 
		filename = new char[strlen(NewName)+1]; 
	      
		CString filenametemp;
		filenametemp=NewName;//.Format("%s",NewName);
		filenametemp.Replace('/','-');
		filenametemp.Replace('>','-');
		filenametemp.Replace('<','-');
		filenametemp.Replace('*','-');
		filenametemp.Replace(':','-');
		filenametemp.Replace('?','-');

		sprintf(filename, "%s", filenametemp);
	} 

	EMULE_CATCH
}
//End by Tarod

Packet*	CKnownFile::CreateSrcInfoPacket(CUpDownClient* forClient)
{
	EMULE_TRY

	CTypedPtrList<CPtrList, CUpDownClient*> srclist;
	theApp.uploadqueue->FindSourcesForFileById(&srclist, forClient->reqfileid); //should we use "filehash"?

	if(srclist.IsEmpty())
		return 0;

	CMemFile data;
	uint16 nCount = 0;

	data.Write(forClient->reqfileid, 16);
	data.Write(&nCount,2);

	uint32 lastRequest = forClient->GetLastSrcReqTime();
	//we are only taking 30 random sources since we can't be sure if they have parts we need
	//this is hard coded because its a temp solution until next(?) version
	srand(time(NULL));
	for(int i = 0; i < 30; i++) {
		int victim = ((rand() >> 7) % srclist.GetSize());
		POSITION pos = srclist.FindIndex(victim);
		CUpDownClient *cur_src = srclist.GetAt(pos);
		// LANCAST (moosetea) - We don't share Lan clients sources as they are private ips/userids
		if(!cur_src->isOnLAN() && !cur_src->HasLowID() && cur_src != forClient)  {
			nCount++;
			uint32 dwID = cur_src->GetUserID();
			uint16 nPort = cur_src->GetUserPort();
			uint32 dwServerIP = cur_src->GetServerIP();
			uint16 nServerPort = cur_src->GetServerPort();
			data.Write(&dwID,4);
			data.Write(&nPort,2);
			data.Write(&dwServerIP,4);
			data.Write(&nServerPort,2);
		}

		srclist.RemoveAt(pos);
		if(srclist.GetSize() == 0)
			break;
	}
	if (!nCount)
		return 0;
	data.Seek(16,0);
	data.Write(&nCount,2);

	Packet* result = new Packet(&data,OP_EMULEPROT);
	result->opcode = OP_ANSWERSOURCES;
	if (nCount > 28)
		result->PackPacket();
	return result;

	EMULE_CATCH

	return NULL;
}

//For File Comment // 
void CKnownFile::LoadComment()
{
	EMULE_TRY

	char buffer[100]; 
	char* fullpath = new char[strlen(theApp.glob_prefs->GetAppDir())+13]; 
	sprintf(fullpath,"%sfileinfo.ini",theApp.glob_prefs->GetAppDir()); 
	   
	buffer[0] = 0;
	for (uint16 i = 0;i != 16;i++) 
		sprintf(buffer,"%s%02X",buffer,filehash[i]); 
	    
	CIni ini( fullpath, buffer); 
	m_strComment = ini.GetString("Comment"); 
	m_iRate = ini.GetInt("Rate", 0);//For rate
	m_bCommentLoaded=true;
	delete fullpath;

	EMULE_CATCH
}    

void CKnownFile::SetFileComment(CString strNewComment)
{
	EMULE_TRY

	char buffer[100]; 
	char* fullpath = new char[strlen(theApp.glob_prefs->GetAppDir())+13]; 
	sprintf(fullpath,"%sfileinfo.ini",theApp.glob_prefs->GetAppDir()); 
	       
	buffer[0] = 0; 
	for (uint16 i = 0;i != 16;i++) 
		sprintf(buffer,"%s%02X",buffer,filehash[i]); 
	    
	CIni ini( fullpath, buffer ); 
	    
	ini.WriteString ("Comment", strNewComment); 
	m_strComment = strNewComment;
	delete fullpath;
	   
	CTypedPtrList<CPtrList, CUpDownClient*> srclist;
	theApp.uploadqueue->FindSourcesForFileById(&srclist, GetFileHash());

	for (POSITION pos = srclist.GetHeadPosition();pos != 0;srclist.GetNext(pos)){
		CUpDownClient *cur_src = srclist.GetAt(pos);
		cur_src->SetCommentDirty();
	}

	EMULE_CATCH
}

// For File rate 
void CKnownFile::SetFileRate(int8 iNewRate)
{
	EMULE_TRY

	char buffer[100]; 
	char* fullpath = new char[strlen(theApp.glob_prefs->GetAppDir())+13]; 
	sprintf(fullpath,"%sfileinfo.ini",theApp.glob_prefs->GetAppDir()); 
	       
	buffer[0] = 0; 
	for (uint16 i = 0;i != 16;i++) 
		sprintf(buffer,"%s%02X",buffer,filehash[i]); 
	    
	CIni ini( fullpath, buffer ); 
	    
	ini.WriteInt ("Rate", iNewRate); 
	m_iRate = iNewRate; 
	delete fullpath;

	CTypedPtrList<CPtrList, CUpDownClient*> srclist;
	theApp.uploadqueue->FindSourcesForFileById(&srclist, GetFileHash());
	for (POSITION pos = srclist.GetHeadPosition();pos != 0;srclist.GetNext(pos)){
		CUpDownClient *cur_src = srclist.GetAt(pos);
		cur_src->SetCommentDirty();
	}

	EMULE_CATCH
}

//--- xrmb:parttraffic ---
void CFileStatistic::initTraffic()
{
	EMULE_TRY

	if(partTraffic==NULL)
	{
		uint16 p=fileParent->GetPartCount();
		partTraffic=new uint32[p];
		partTrafficSession=new uint32[p];
		for(uint16 i=0; i<p; i++)
		{
			partTraffic[i]=0;
			partTrafficSession[i]=0;
		}
	}

	if(partAccepted==NULL)
	{
		uint16 p=fileParent->GetPartCount();
		partAccepted=new uint32[p];
		partAcceptedSession=new uint32[p];
		for(uint16 i=0; i<p; i++)
		{
			partAccepted[i]=0;
			partAcceptedSession[i]=0;
		}
	}

	if(blockTraffic==NULL)
	{
		uint32 p=fileParent->GetBlockCount();
		blockTraffic=new uint32[p];
		blockTrafficSession=new uint32[p];
		for(uint32 i=0; i<p; i++)
		{
			blockTraffic[i]=0;
			blockTrafficSession[i]=0;
		}
	}

	EMULE_CATCH
}

void CFileStatistic::recalcCompleteReleases()
{
	EMULE_TRY

	//--- recalculate complete releases factor ---
	uint32 blocks=fileParent->GetBlockCount();
	uint32 fs=fileParent->GetFileSize();

	//--- find lowest parttraffic ---
	uint32 m=0xFFFFFFFF;
	completeReleases=0;
	for(uint32 i=0; i<blocks; i++)
	{
		if(blockTraffic[i]==0)
			continue;

		uint32 s=BLOCKSIZE;
		if(BLOCKSIZE*(i+1) > fs) 
			s = fs % BLOCKSIZE; 
		
		if(s)
		{
			uint32 f=(uint32)(double)blockTraffic[i]/s+(blockTraffic[i]==s?0:1);
			if(f<m) m=f;
		}
	}

	//--- get avg. ---
	for(uint32 i=0; i<blocks; i++)
	{
		if(blockTraffic[i]==0)
			continue;

		uint32 s=BLOCKSIZE;
		if(BLOCKSIZE*(i+1) > fs) 
			s = fs % BLOCKSIZE;

		if(s)
		{
			double f=(double)blockTraffic[i]/s;
			if(f>m) f=m;
			completeReleases+=f;
		}
	}

	completeReleases/=blocks;

	EMULE_CATCH
}

//--- this function may not return a 100% accurate value, because you can't fill a part with blocks
double CFileStatistic::GetCompletePartReleases(uint16 part)
{
	EMULE_TRY

	//--- check part, damn that all needs time, why we just cant trust the devs :) ---
	if(partTraffic==NULL || part<0 || part>fileParent->GetPartCount() || partTraffic[part]==0)
		return 0;

	//--- start and end block ---
	uint32	startBlock=(part*PARTSIZE)/BLOCKSIZE;
	uint32	endBlock=((part+1)*PARTSIZE)/BLOCKSIZE;
	if(endBlock >= fileParent->GetBlockCount())
		endBlock=fileParent->GetBlockCount()-1;

	uint32	fs=fileParent->GetFileSize();

	//--- find lowest parttraffic ---
	uint32	m=0xFFFFFFFF;
	double	completePartReleases=0;
	for(uint32 i=startBlock; i<=endBlock; i++)
	{
		if(blockTraffic[i]==0)
			continue;

		uint32 s=BLOCKSIZE;
		if(BLOCKSIZE*(i+1) > fs) 
			s = fs % BLOCKSIZE; 
		
		if(s)
		{
			uint32 f=(uint32)(double)blockTraffic[i]/s+(blockTraffic[i]==s?0:1);
			if(f<m) m=f;
		}
	}

	//--- get avg. ---
	for(uint32 i=startBlock; i<=endBlock; i++)
	{
		if(blockTraffic[i]==0)
			continue;

		uint32 s=BLOCKSIZE;
		if(BLOCKSIZE*(i+1) > fs) 
			s = fs % BLOCKSIZE;

		if(s)
		{
			double f=(double)blockTraffic[i]/s;
			if(f>m) f=m;
			completePartReleases+=f;
		}
	}

	return completePartReleases/(fileParent->GetPartSize(part)/BLOCKSIZE);

	EMULE_CATCH

	return 0;
}

void CFileStatistic::AddTraffic(uint32 start, uint32 bytes)
{
	EMULE_TRY

	initTraffic();

	//DEBUG_ONLY(theApp.emuledlg->AddLogLine(false, "--------------------------------------------------------------------------------------------------"));
	//DEBUG_ONLY(theApp.emuledlg->AddLogLine(false, "start %i, bytes %i ", start, bytes));

	//--- start block ---
	uint32	part=start/PARTSIZE;
	sint32	btg=(sint32)bytes;

	uint32 p1=min((part+1)*PARTSIZE-start, bytes);
	partTraffic[part]+=p1;
	partTrafficSession[part]+=p1;
	partAccepted[part]++;
	partAcceptedSession[part]++;
	btg-=p1;
	part++;
	while(btg>0)
	{
		partTraffic[part]+=btg % PARTSIZE;
		partTrafficSession[part]+=btg % PARTSIZE;
		partAccepted[part]++;
		partAcceptedSession[part]++;
		part++;
		btg-=PARTSIZE;
	}

	//--- start block ---
	uint32	block=start/BLOCKSIZE;
	btg=(sint32)bytes;

	uint32 b1=min((block+1)*BLOCKSIZE-start, bytes);
	blockTraffic[block]+=b1;
	blockTrafficSession[block]+=b1;
	btg-=b1;
	block++;
	while(btg>0)
	{
		blockTraffic[block]+=btg % BLOCKSIZE;
		blockTrafficSession[block]+=btg % BLOCKSIZE;
		block++;
		btg-=BLOCKSIZE;
	}

	recalcCompleteReleases();
	
	theApp.sharedfiles->UpdateItem(fileParent);

	EMULE_CATCH
}

bool CKnownFile::SaveToFileTraffic(FILE* file)
{
	EMULE_TRY

	if(statistic.partTraffic)
	{
		uint16 parts=GetPartCount();
		if(fwrite(&parts, sizeof(parts), 1, file)!=1) return false;
		if(fwrite(statistic.partTraffic, 4, parts, file)!=parts) return false;

		uint32 blocks=GetBlockCount();
		if(fwrite(&blocks, sizeof(blocks), 1, file)!=1) return false;
		if(fwrite(statistic.blockTraffic, 4, blocks, file)!=blocks) return false;

		//--- added in version 2 ---
		if(fwrite(statistic.partAccepted, 4, parts, file)!=parts) return false;
	}

	//--- this block shouldnt be used at all ---
	else
	{
		uint32	d=0;

		uint16 parts=GetPartCount();
		if(fwrite(&parts, sizeof(parts), 1, file)!=1) return false;
		for(uint16 part=0; part<parts; part++)
			if(fwrite(&d, sizeof(d), 1, file)!=1) return false;

		uint32 blocks=GetBlockCount();
		if(fwrite(&blocks, sizeof(blocks), 1, file)!=1) return false;
		for(uint32 block=0; block<blocks; block++)
			if(fwrite(&d, sizeof(d), 1, file)!=1) return false;

		//--- added version 2 ---
		for(uint16 part=0; part<parts; part++)
			if(fwrite(&d, sizeof(d), 1, file)!=1) return false;
	}

	return ferror(file);

	EMULE_CATCH

	return false;
}

bool CKnownFile::LoadFromFileTraffic(FILE* file, uint8 version)
{
	EMULE_TRY

	// read parts
	uint16 parts=0;
	if(fread(&parts, sizeof(parts), 1, file)!=1) return false;
	
	// does part count match?
	if(parts!=GetPartCount())
	{
		theApp.emuledlg->AddLogLine(true, "traffic.dat is corrupted");
		return false;
	}

	// get the memory
	statistic.initTraffic();

	// load all the parts traffic data
	if(fread(statistic.partTraffic, 4, parts, file)!=parts) return false;

	// read blocks
	uint32 blocks=0;
	if(fread(&blocks, sizeof(blocks), 1, file)!=1) return false;
	
	// does blocks count match?
	if(blocks!=GetBlockCount())
	{
		theApp.emuledlg->AddLogLine(true, "traffic.dat is corrupted");
		return false;
	}

	// load all the parts traffic data
	if(fread(statistic.blockTraffic, 4, blocks, file)!=blocks) return false;

	//--- versioning ---
	if(version>1)
	{
		//--- version 2 introduced part-accepted counter ---
		if(fread(statistic.partAccepted, 4, parts, file)!=parts) return false;
	}

	//--- recalc something ---
	statistic.recalcCompleteReleases();

	return true;

	EMULE_CATCH

	return false;
}

uint32 CKnownFile::GetBlockCount()
{
	EMULE_TRY

	return (filesize/BLOCKSIZE) + ((filesize % BLOCKSIZE)? 1:0);

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetTrafficBlock(uint32 start, uint32 end, bool session)
{
	EMULE_TRY

	if(statistic.blockTraffic==NULL)
		return 0;

	if(start>filesize) return 0;
	if(end>filesize) end=filesize;

	uint32	*data;
	if(session)
		data=statistic.blockTrafficSession;
	else
		data=statistic.blockTraffic;

	uint32	block=start/BLOCKSIZE;
	uint32	pos=start;
	sint32	bftb;
	uint32	t=0;
	while(pos < end)
	{
		if(end-pos > end % BLOCKSIZE)
			bftb=BLOCKSIZE;
		else
			bftb=end % BLOCKSIZE;
		bftb-=pos % BLOCKSIZE;

		if(bftb != BLOCKSIZE)
			t+=data[block] * (double)bftb/GetBlockSize(block);
		else
			t+=data[block];

		pos+=bftb;

		block++;
	}

	return t;

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetTrafficPart(uint32 start, uint32 end, bool session)
{
	EMULE_TRY

	if(statistic.partTraffic==NULL)
		return 0;

	if(start>filesize) return 0;
	if(end>filesize) end=filesize;

	uint32	*data;
	if(session)
		data=statistic.partTrafficSession;
	else
		data=statistic.partTraffic;

	uint32	part=start/PARTSIZE;
	uint32	pos=start;
	sint32	bftp;
	uint32	t=0;
	while(pos < end)
	{
		if(end-pos > end % PARTSIZE)
			bftp=PARTSIZE;
		else
			bftp=end % PARTSIZE;
		bftp-=pos % PARTSIZE;

		if(bftp != PARTSIZE)
			t+=data[part] * (double)bftp/GetPartSize(part);
		else
			t+=data[part];

		pos+=bftp;

		part++;
	}

	return t;

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetPartTraffic(uint16 part, bool session)
{
	EMULE_TRY

	if(statistic.partTraffic==NULL)
		return 0;

	if(session)
		return statistic.partTrafficSession[part];
	else
		return statistic.partTraffic[part];

	EMULE_CATCH
}

uint32 CKnownFile::GetBlockTraffic(uint16 block, bool session)
{
	EMULE_TRY

	if(statistic.partTraffic==NULL)
		return 0;

	if(session)
		return statistic.blockTrafficSession[block];
	else
		return statistic.blockTraffic[block];

	EMULE_CATCH
}

uint32 CKnownFile::GetPartSize(uint16 part)
{
	EMULE_TRY

	if(part+1<GetPartCount())
		return PARTSIZE;

	return filesize % PARTSIZE;

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetBlockSize(uint32 block)
{
	EMULE_TRY

	if(block+1<GetBlockCount())
		return BLOCKSIZE;

	return filesize % BLOCKSIZE;

	EMULE_CATCH

	return 0;
}

void CFileStatistic::resetStats(bool all)
{
	EMULE_TRY

	if(all)
	{
		if(partTraffic)		delete[] partTraffic;	partTraffic=NULL;
		if(partAccepted)	delete[] partAccepted;	partAccepted=NULL;
		if(blockTraffic)	delete[] blockTraffic;	blockTraffic=NULL;
	
		if(partTrafficSession)		delete[] partTrafficSession;	partTrafficSession=NULL;
		if(partAcceptedSession)		delete[] partAcceptedSession;	partAcceptedSession=NULL;
		if(blockTrafficSession)		delete[] blockTrafficSession;	blockTrafficSession=NULL;

		completeReleases=0.0;

		alltimerequested=alltimetransfered=alltimeaccepted=0;
	}
	else
	{
		if(partTrafficSession)
			for(uint16 part=0; part<fileParent->GetPartCount(); part++)
				partTrafficSession[part]=0;

		if(partAcceptedSession)
			for(uint16 part=0; part<fileParent->GetPartCount(); part++)
				partAcceptedSession[part]=0;

		if(blockTrafficSession)
			for(uint32 block=0; block<fileParent->GetBlockCount(); block++)
				blockTrafficSession[block]=0;
	}
	
	requested=transfered=accepted=0;

	EMULE_CATCH
}

void CFileStatistic::resetPartTraffic(uint16 part, bool all)
{
	EMULE_TRY

	if(partTraffic==NULL || part>=fileParent->GetPartCount())
		return;

	//--- this all is a little bit tricky, because the part and block size have no connections ---
	//--- so i have to remove some bytes only from the start and end block ---
	//--- all we try is to keep it as realistic as possible :) ---
	uint32	blocks=fileParent->GetBlockCount();
	uint32	blockStart=(part*PARTSIZE)/BLOCKSIZE;
	uint32	blockEnd=min((((uint32)part+1)*PARTSIZE)/BLOCKSIZE+1, blocks-1);
	float	percentStart=((float)((part*PARTSIZE)%BLOCKSIZE)/BLOCKSIZE);
	float	percentEnd=0.0;
	if(blockEnd!=blocks)
		percentEnd=1-((float)(((part+1)*PARTSIZE)%BLOCKSIZE)/BLOCKSIZE);

	if(all)
	{
		partTraffic[part]=0;
		blockTraffic[blockStart]*=percentStart;
		blockTraffic[blockEnd]*=percentEnd;
		for(uint32 block=blockStart+1; block<blockEnd; block++)
			blockTraffic[block]=0;
	}

	partTrafficSession[part]=0;
	blockTrafficSession[blockStart]*=percentStart;
	blockTrafficSession[blockEnd]*=percentEnd;
	for(uint32 block=blockStart+1; block<blockEnd; block++)
		blockTrafficSession[block]=0;

	EMULE_CATCH
}

uint32	CFileStatistic::GetPartAccepted(uint16 part, bool session) const
{
	EMULE_TRY

	if(partTraffic==NULL || part>=fileParent->GetPartCount())
		return 0;

	if(session==false)
		return partAccepted[part];
	else
		return partAcceptedSession[part];

	EMULE_CATCH

	return 0;
}
//--- :xrmb ---

//--- xrmb:keepPartFileStats ---
void CFileStatistic::AddAll(uint32 _alltimerequested, uint32 _alltimeaccepted, uint64 _alltimetransfered)
{
	alltimerequested	+= _alltimerequested;
	alltimeaccepted		+= _alltimeaccepted;
	alltimetransfered	+= _alltimetransfered;
}
//--- :xrmb ---

//--- xrmb:allyaknow ---
//-------------------------------------------------------------------------------------------------
bool	CFileStatistic::merge(CFileStatistic *m)
{
	EMULE_TRY

	alltimerequested	+= m->alltimerequested;
	alltimeaccepted		+= m->alltimeaccepted;
	alltimetransfered	+= m->alltimetransfered;

	requested	+= m->requested;
	accepted	+= m->accepted;
	transfered	+= m->transfered;

	//--- merge parttraffic ---

	//--- i dont have traffic, but the to merge has ---
	if(partTraffic==NULL && m->partTraffic)
		initTraffic();

	//--- merge traffic only if the to merge has some ---
	if(m->partTraffic)
	{
		for(uint16 part=0; part<fileParent->GetPartCount(); part++)
			partTraffic[part]=m->partTraffic[part];

		for(uint32 block=0; block<fileParent->GetBlockCount(); block++)
			blockTraffic[block]+=m->blockTraffic[block];
	}

	return true;

	EMULE_CATCH

	return false;
}
//--- :xrmb ---

//--- xrmb:partprio ---
uint8	CKnownFile::GetPartStatus(uint16 part)
{ 
	EMULE_TRY

	uint8 r=PR_PART_ON; 
	if(m_partStatus.Lookup(part, r)) 
		return r; 

	EMULE_CATCH

	return PR_PART_ON; 
}

bool	CKnownFile::HasHiddenParts()
{
	EMULE_TRY

	POSITION	pos=m_partStatus.GetStartPosition();
	uint16		part;
	uint8		status;
	while(pos)
	{
		m_partStatus.GetNextAssoc(pos, part, status);
		if(status!=PR_PART_ON)
			return true;
	}

	EMULE_CATCH

	return false;
}

void CKnownFile::WritePartStatus(CFile* file)
{
	EMULE_TRY

	uint16 parts = hashlist.GetCount();
	file->Write(&parts, 2);
	uint16 done = 0;
	while(done!=parts)
	{
		uint8 towrite = 0;
		for (uint32 i = 0;i != 8;i++){
			if (GetPartStatus(done)==PR_PART_ON)
				towrite |= (1<<i);

			done++;
			if (done == parts)
				break;
		}
		file->Write(&towrite,1);
	}

	EMULE_CATCH
}
//--- :xrmb/partprio ---



// obaldin
#if JUMPSTART
void CKnownFile::SetJumpstartEnabled(bool enabled) {
    if(!enabled && m_Jumpstarter) {
        delete m_Jumpstarter;
        m_Jumpstarter = NULL;
    } else if(enabled && !m_Jumpstarter) {
        CJumpstarter::EnableForFile(this);
        m_Jumpstarter = new CJumpstarter(this);
    }
}
void CKnownFile::AddSentBlock(CUpDownClient* client,uint32 start_offset, uint32 togo) {
    m_Jumpstarter->AddSentBlock(client, start_offset, togo);
}

void CKnownFile::WriteJumpstartPartStatus(CUpDownClient* client, CMemFile* data) {
    m_Jumpstarter->WriteJumpstartPartStatus(client, data);
}
#endif
